package cn.aid.resourceservice.service;

import cn.aid.common.utils.api.model.APICode;
import cn.aid.common.utils.api.model.APIResult;
import cn.aid.common.utils.api.page.Page;
import cn.aid.common.utils.constant.ResourceType;
import cn.aid.common.utils.dto.resource.ResourceSearchDTO;
import cn.aid.common.utils.enums.BaseOrderEnum;
import cn.aid.common.utils.exception.DuplicateValueException;
import cn.aid.common.utils.exception.NotFoundException;
import cn.aid.common.utils.exception.UnsupportedOperationException;
import cn.aid.data.api.resource.entity.Resource;
import cn.aid.resourceservice.dto.ResourceDetail;
import cn.aid.resourceservice.dto.ResourceRemote;
import cn.aid.resourceservice.dto.ResourceVideo;
import cn.aid.resourceservice.repository.ResourceRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;


@Slf4j
@Service
public class ResourceService {

    @Autowired
    private ResourceRepository resourceRepository;



    @Value("${resource.img.domain}")
    private String imgDomain;

    @Value("${resource.audio.domain}")
    private String audioDomain;

    @Value("${resource.video.domain}")
    private String videoDomain;

    public boolean exists(String code) {
        Long count = resourceRepository.countByCode(code);
        return count > 0L;
    }

    public Resource get(String id) {
        return resourceRepository.find(id);
    }

    public Resource save(Resource resource) {
        return resourceRepository.save(resource);
    }

    public APIResult<Page<Resource>> search(Resource resource, Integer pageNo, Integer pageSize) {
        Long count = resourceRepository.count(resource);
        Page<Resource> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Resource> resourceList = resourceRepository.find(resource, page.getStart(), pageSize, new Sort(Sort.Direction.DESC, "gmtModified"));
        resourceList.forEach(r -> {
            initResourceUrl(r);
        });

        page.setList(resourceList);

        return APIResult.ok(page);
    }

    public APIResult<Map<String, Resource>> findMixByIds(List<String> ids) {
        Map<String, Resource> resourceMap = new HashMap<>();

        // 取本地
        List<Resource> nativeResourceList = resourceRepository.findByIds(ids);
        nativeResourceList.forEach(resource -> {
            initResourceUrl(resource);
            resourceMap.put(resource.getId(), resource);
        });

        // 本地基础资源取够了不取了
        if (nativeResourceList.size() == ids.size() && nativeResourceList.size() > 0) {
            return APIResult.ok(resourceMap);
        }

        // 取本地,有声读物资源



        // 本地取够了不取了
        if (nativeResourceList.size() == ids.size() && nativeResourceList.size() > 0) {
            return APIResult.ok(resourceMap);
        }



        return APIResult.ok(resourceMap);
    }





    public void initResourceUrl(Resource resource) {
        if (resource == null) {
            return;
        }else if (resource.getPath() == null) {
            return;
        }

        String path = resource.getPath();
        if (ResourceType.VIDEO == resource.getType()) {
            resource.setUrl(concatUrl(videoDomain, path));
        }else if (ResourceType.AUDIO == resource.getType()) {
            resource.setUrl(concatUrl(audioDomain, path));
        }else if (ResourceType.IMG == resource.getType()) {
            resource.setUrl(concatUrl(imgDomain, path).replace(" ", ""));
        }
    }

    public static String concatUrl(String domain, String path) {
        if (StringUtils.isEmpty(domain) || StringUtils.isEmpty(path)) {
            return null;
        }
        if (path.startsWith("/") && domain.endsWith("/")) {
            return domain.substring(0, domain.length()) + path;
        }else if(path.startsWith("/") || domain.endsWith("/")){
            return domain + path;
        }else {
            return domain + "/" + path;
        }
    }

    /**
     *
     * @param resource
     * @throws DuplicateValueException
     * @throws NotFoundException
     */
    public Resource update(Resource resource) throws DuplicateValueException, NotFoundException, UnsupportedOperationException {
        if (null == resource){
            log.error("Resource is null");
            return null;
        }

        if (ResourceType.IMG == resource.getType() || ResourceType.AUDIO == resource.getType() ){
            return updateResource(resource);
        }
        else{
            log.error("Not-Supported-Resource-Type, id={}, type={}", resource.getId(), resource.getType());
            throw new UnsupportedOperationException("Not-Supported-Resource-Type");
        }
    }

    /**
     * update Img
     * @param resource
     * @throws DuplicateValueException
     * @throws NotFoundException
     */
    private void updateImg(Resource resource) throws DuplicateValueException, NotFoundException {

        if (ResourceType.IMG != resource.getType()){
            log.error("Not-Img-Resource");
            return;
        }

        //find target resource
        Resource targetResouce = resourceRepository.find(resource.getId());

        if(null == targetResouce){
            log.error("Cannot Find Resource, id={}", resource.getId());
            throw new NotFoundException("Resource-Not-Found");
        }

        //only support code,title,path
        //first, make sure code is not duplicated
        List<Resource> foundedResource = resourceRepository.findByCode(resource.getCode());
        foundedResource.forEach( (each) -> {
            if ( !resource.getId().equals(each.getId()) ){
                log.error("Code-Already-Exist, code={}", resource.getCode());
                throw new DuplicateValueException("Code-Already-Exist");
            }
        });

        //set and save
        targetResouce.setCode(resource.getCode());
        targetResouce.setName(resource.getName());
        targetResouce.setPath(resource.getPath());
        resourceRepository.save(targetResouce);
    }

    /**
     * update Img,audio
     * @param resource
     * @throws DuplicateValueException
     * @throws NotFoundException
     */
    public Resource updateResource(Resource resource) throws DuplicateValueException, NotFoundException {

        if (ResourceType.IMG != resource.getType() && ResourceType.AUDIO!= resource.getType()){
            log.error("Not-Img/audio -Resource");
            return null;
        }

        //find target resource
        Resource targetResouce = resourceRepository.find(resource.getId());

        if(null == targetResouce){
            log.error("Cannot Find Resource, id={}", resource.getId());
            throw new NotFoundException("Resource-Not-Found");
        }

        //only support code,title,path
        //first, make sure code is not duplicated
        List<Resource> foundedResource = resourceRepository.findByCode(resource.getCode());
        foundedResource.forEach( (each) -> {
            if ( !resource.getId().equals(each.getId()) ){
                log.error("Code-Already-Exist, code={}", resource.getCode());
                throw new DuplicateValueException("Code-Already-Exist");
            }
        });

        //set and save
        targetResouce.setCode(resource.getCode());
        targetResouce.setName(resource.getName());
        targetResouce.setPath(resource.getPath());
        return resourceRepository.save(targetResouce);
    }

    /**
     * saveRemoteResource
     * type 0 video , 1 audio
     * @param resource
     * @return
     */
    public APIResult<Resource> saveRemoteResource(Resource resource) {

        Set<ResourceVideo> videos = new HashSet<>();
        ResourceVideo video = new ResourceVideo();
        video.setBucket("efunbox");
        video.setPath(resource.getPath());
        video.setQuality(resource.getQuality());
        video.setFormat(resource.getFormat());
        video.setSize(resource.getSize());
        videos.add(video);


        ResourceRemote resourceRemote = new ResourceRemote();
        resourceRemote.setNo(resource.getCode());
        resourceRemote.setTitle(resource.getName());
        resourceRemote.setType(resource.getType());
        resourceRemote.setVideos(videos);




//        Resource resourceVO = apiResult.getData().toNativeResource();
        return APIResult.ok();
    }

    /**
     * update  RemoteResource
     * @param resource
     * @return
     */
    public APIResult<Resource> updateRemoteResource(Resource resource) {

        if (org.apache.commons.lang.StringUtils.isEmpty(resource.getId())) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        Set<ResourceVideo> videos = new HashSet<>();
        ResourceVideo video = new ResourceVideo();
        video.setId(resource.getId());
        video.setBucket("efunbox");
        video.setPath(resource.getPath());
        video.setQuality(resource.getQuality());
        video.setFormat(resource.getFormat());
        video.setSize(resource.getSize());
        videos.add(video);


        ResourceRemote resourceRemote = new ResourceRemote();
        resourceRemote.setId(resource.getRid());
        resourceRemote.setNo(resource.getCode());
        resourceRemote.setTitle(resource.getName());
        resourceRemote.setType(resource.getType());
        resourceRemote.setVideos(videos);



        return APIResult.ok();
    }

    private APIResult<Resource> errorResourceAPIResult(APIResult<ResourceDetail> apiResult) {
        log.error("Remote Server Error: code={}, message={}", apiResult.getCode(), apiResult.getMessage());
        APIResult errResult = APIResult.error(APICode.REMOTE_SERVER_ERROR);
        errResult.setMessage(apiResult.getMessage());
        return errResult;
    }



}
